package com.caimei365.order.components;

import com.alibaba.fastjson.JSON;
import com.caimei365.order.constant.Constant;
import com.caimei365.order.mapper.OrderCommonMapper;
import com.caimei365.order.mapper.PayOrderMapper;
import com.caimei365.order.model.bo.VipRecordBo;
import com.caimei365.order.model.dto.HeliDto;
import com.caimei365.order.model.enums.BizType;
import com.caimei365.order.model.vo.*;
import com.caimei365.order.utils.MathUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

@Slf4j
@Service
public class HeliPayUtil {

    @Resource
    private PayOrderMapper payOrderMapper;
    @Resource
    private OrderCommonMapper orderCommonMapper;
    @Value("${pay.second-notify-url}")
    private String secondHandUrl;
    @Value("${pay.vip-notify-url}")
    private String superVipUrl;
    @Value("${pay.auth-vip-notify-url}")
    private String authVipUrl;
    @Value("${wx.mini-app-id}")
    private String miniAppId;
    @Value("${pay.notify-url}")
    private String notifyUrl;
    @Value("${pay.union-url}")
    private String unionPay;
    @Value("${pay.vip-union-url}")
    private String vipUnionPay;
    @Value("${pay.auth-vip-union-url}")
    private String authVipUnionPay;
    @Value("${pay.second-union-url}")
    private String secondUnionPay;
    @Value("${pay.coupon-notify-url}")
    private String couponUrl;
    @Value("${pay.coupon-union-url}")
    private String couponUnionPay;

    private final String publicCode="wx91c4152b60ca91a3";

    //小程序/公众号线上公用方法 p8 openid在自己方法传
    public AppPayPublicCreateOrderVo setOnlineValue(AppPayPublicCreateOrderVo pay, HeliDto heliDto, String payFlag, HttpHeaders headers) {
        String orderId = "";
        // 时间戳
        long time = System.currentTimeMillis() / 1000;
        String environment = "";
        if (secondHandUrl.contains("18002")) {
            environment = "DEV";
        } else if (secondHandUrl.contains("core-b")) {
            environment = "BETA";
        }
        if ("second".equals(payFlag)) {
            //二手支付固定100
            pay.setP9_orderAmount("100");
            //二手
            orderId = heliDto.getProductId() + "T" + time + environment;
            pay.setP12_notifyUrl(secondHandUrl);
            String product = "采美订单" + orderId;
            pay.setP15_goodsName(product);
            //备注
            pay.setP18_desc(heliDto.getProductId().toString() + "," + BizType.getNumByType(heliDto.getPayType()));
            //收款账号
            pay.setP3_customerNumber(Constant.CUSTOMERNUM);
            //立即分账，固定金额
//            pay.setSplitBillType("FIXED_AMOUNT");
            //分账对象
//            PaySplitVo paySplitVo = new PaySplitVo();
            //分账账户邮箱 二手进信息
            //交易分账规则串必须包含两个参数，分别是分账商户的合法邮箱splitBillMerchantEmail和对应分账合法金额splitBillAmount，
            //分账金额单位:元,最多保留两位小数，分账规则串中不可以重复出现同一邮箱，分账规则串中不可以传分账发起方自己；
//            paySplitVo.setSplitBillMerchantEmail(Constant.XX_MAIL);
            //手续费0.25% 支付金额*(1-0.25%)
//            paySplitVo.setSplitBillAmount("99.75");
//            String s = JSON.toJSONString(new ArrayList<PaySplitVo>().add(paySplitVo));
//            pay.setRuleJson(s);
        } else if ("vip".equals(payFlag)) {
            VipRecordBo record = payOrderMapper.getVipPackageRecord(heliDto.getVipRecordId());
            Double price = payOrderMapper.getVipPackagePriceById(record.getPackageId());
            //会员
            orderId = record.getUserId() + "R" + heliDto.getVipRecordId() + "V" + record.getPackageId() + "T" + time + environment;
            pay.setP9_orderAmount(price.toString());
            pay.setP12_notifyUrl(superVipUrl);
            String product = "采美订单" + orderId;
            pay.setP15_goodsName(product);
            String attach = record.getPackageId() + "," + record.getUserId() + "," + heliDto.getVipRecordId() + "," + BizType.getNumByType(heliDto.getPayType());
            pay.setP18_desc(attach);
            //收款账号
            pay.setP3_customerNumber(Constant.CUSTOMERNUM);
        } else if ("authVip".equals(payFlag)) {
            VipRecordBo record = payOrderMapper.getAuthVipRecord(heliDto.getVipRecordId());
            Double price = payOrderMapper.getAuthVipPackagePrice(record.getPackageId());
            //会员
            orderId = record.getUserId() + "A" + heliDto.getVipRecordId() + "V" + record.getPackageId() + "T" + time + environment;
            pay.setP9_orderAmount(price.toString());
            pay.setP12_notifyUrl(authVipUrl);
            String product = "采美订单" + orderId;
            pay.setP15_goodsName(product);
            String attach = record.getPackageId() + "," + record.getUserId() + "," + heliDto.getVipRecordId() + "," + BizType.getNumByType(heliDto.getPayType());
            pay.setP18_desc(attach);
            //收款账号
            pay.setP3_customerNumber(Constant.CUSTOMERNUM);
        }else if("coupon".equals(payFlag)){
            Double couponPrice = payOrderMapper.getCouponPrice(heliDto.getCouponRecordId());
            orderId = heliDto.getUserId() + "C" + heliDto.getCouponId() + "R" + heliDto.getCouponRecordId() + "T" + time + environment;
            pay.setP9_orderAmount(couponPrice.toString());
            pay.setP12_notifyUrl(couponUrl);
            String product = "采美订单" + orderId;
            pay.setP15_goodsName(product);
            String attach = heliDto.getCouponId() + "," + heliDto.getUserId() + "," + heliDto.getCouponRecordId() + "," + BizType.getNumByType(heliDto.getPayType())+","+heliDto.getSource();
            pay.setP18_desc(attach);
            //收款账号
            pay.setP3_customerNumber(Constant.CUSTOMERNUM);
        } else if ("order".equals(payFlag)) {
            //普通订单
            OrderVo order = orderCommonMapper.getOrderByShopOrderId(heliDto.getShopOrderId());
            orderId = order.getOrderNo() + "T" + time + environment;
            pay.setP9_orderAmount(heliDto.getPayAmount());
            pay.setP12_notifyUrl(notifyUrl);
            String product = "采美订单" + order.getOrderNo();
            pay.setP15_goodsName(product);
            String attach = order.getOrderId() + "," + BizType.getNumByType(heliDto.getPayType())+","+heliDto.getShopOrderId();
            pay.setP18_desc(attach);
        }
        pay.setP1_bizType(BizType.getP1ByType(heliDto.getPayType()));
        pay.setP2_orderId(orderId);
        pay.setP4_payType(BizType.getP4ByType(heliDto.getPayType()));
        pay.setP10_currency("CNY");
        pay.setP14_orderIp(headers.getFirst("X-CLIENT-IP"));
        pay.setP11_appType("WXPAY");
        //成功后跳转地址
        pay.setP13_successToUrl(heliDto.getReturnUrl());
        if ("GZH".equals(heliDto.getPayType())) {
            //公众号appid
            pay.setP5_appid(publicCode);
        } else if ("XCX".equals(heliDto.getPayType())) {
            //小程序appid
            pay.setP5_appid(miniAppId);
        }
        return pay;
    }

    //pc支付宝，微信二维码设值
    public AppCreateOrderVo setValue(AppCreateOrderVo pay, HeliDto heliDto, String payFlag, HttpHeaders headers) {
        String orderId = "";
        // 时间戳
        long time = System.currentTimeMillis() / 1000;
        String environment = "";
        if (secondHandUrl.contains("18002")) {
            environment = "DEV";
        } else if (secondHandUrl.contains("core-b")) {
            environment = "BETA";
        }
        if ("second".equals(payFlag)) {
            pay.setP5_orderAmount("100");
            pay.setP9_notifyUrl(secondHandUrl);
            orderId = heliDto.getProductId() + "T" + time + environment;
            String product = "采美订单" + orderId;
            pay.setP12_goodsName(product);
            pay.setP14_desc(heliDto.getProductId().toString() + "," + BizType.getNumByType(heliDto.getPayType()));
            //网络收款，分账进信息
            pay.setP3_customerNumber(Constant.CUSTOMERNUM);
        }else if ("vip".equals(payFlag)) {
            VipRecordBo record = payOrderMapper.getVipPackageRecord(heliDto.getVipRecordId());
            Double price = payOrderMapper.getVipPackagePriceById(record.getPackageId());
            pay.setP5_orderAmount(price.toString());
            pay.setP9_notifyUrl(superVipUrl);
            orderId = record.getUserId() + "R" + heliDto.getVipRecordId() + "V" + record.getPackageId() + "T" + time + environment;
            String product = "采美订单" + orderId;
            pay.setP12_goodsName(product);
            String attach = record.getPackageId() + "," + record.getUserId() + "," + heliDto.getVipRecordId() + "," + BizType.getNumByType(heliDto.getPayType());
            pay.setP14_desc(attach);
            //收款账号
            pay.setP3_customerNumber(Constant.CUSTOMERNUM);
        }else if ("authVip".equals(payFlag)) {
            VipRecordBo record = payOrderMapper.getAuthVipRecord(heliDto.getVipRecordId());
            Double price = payOrderMapper.getAuthVipPackagePrice(record.getPackageId());
            pay.setP5_orderAmount(price.toString());
            pay.setP9_notifyUrl(authVipUrl);
            orderId = record.getUserId() + "A" + heliDto.getVipRecordId() + "V" + record.getPackageId() + "T" + time + environment;
            String product = "采美订单" + orderId;
            pay.setP12_goodsName(product);
            String attach = record.getPackageId() + "," + record.getUserId() + "," + heliDto.getVipRecordId() + "," + BizType.getNumByType(heliDto.getPayType());
            pay.setP14_desc(attach);
            //收款账号
            pay.setP3_customerNumber(Constant.CUSTOMERNUM);
        }else if("coupon".equals(payFlag)){
            Double couponPrice = payOrderMapper.getCouponPrice(heliDto.getCouponRecordId());
            orderId = heliDto.getUserId() + "C" + heliDto.getCouponId() + "R" + heliDto.getCouponRecordId() + "T" + time + environment;
            pay.setP5_orderAmount(couponPrice.toString());
            pay.setP9_notifyUrl(couponUrl);
            String product = "采美订单" + orderId;
            pay.setP12_goodsName(product);
            String attach = heliDto.getCouponId() + "," + heliDto.getUserId() + "," + heliDto.getCouponRecordId() + "," + BizType.getNumByType(heliDto.getPayType())+","+heliDto.getSource();
            pay.setP14_desc(attach);
            //收款账号
            pay.setP3_customerNumber(Constant.CUSTOMERNUM);
        }
        else if ("order".equals(payFlag)) {
            //以子订单为单位付款，商户号为商品的分帐号
            OrderVo order = orderCommonMapper.getOrderByShopOrderId(heliDto.getShopOrderId());
            orderId = order.getOrderNo() + "T" + time + environment;
            pay.setP5_orderAmount(heliDto.getPayAmount());
            pay.setP9_notifyUrl(notifyUrl);
            String product = "采美订单" + order.getOrderNo();
            pay.setP12_goodsName(product);
            String attach = order.getOrderId() + "," + BizType.getNumByType(heliDto.getPayType())+","+heliDto.getShopOrderId();;
            pay.setP14_desc(attach);
        }
        pay.setP1_bizType(BizType.getP1ByType(heliDto.getPayType()));
        pay.setP2_orderId(orderId);
        pay.setP4_payType("SCAN");
        pay.setP6_currency("CNY");
        pay.setP7_authcode("1");
        //pc二维码apptype不是微信就是支付宝
        String appType="WXEWM".equals(heliDto.getPayType())?"WXPAY":"ALIPAY";
        pay.setP8_appType(appType);
        pay.setP10_successToUrl(heliDto.getReturnUrl());
        pay.setP11_orderIp(headers.getFirst("X-CLIENT-IP"));
        return pay;
    }

    public HeliOnlineVo setLinkValue(HeliOnlineVo pay, HeliDto heliDto, String payFlag, HttpHeaders headers) {
        try {
            String orderId = "";
            // 时间戳
            long time = System.currentTimeMillis() / 1000;
            String environment = "";
            if (secondHandUrl.contains("18002")) {
                environment = "DEV";
            } else if (secondHandUrl.contains("core-b")) {
                environment = "BETA";
            }
            if ("second".equals(payFlag)) {
                pay.setP3_customerNumber(Constant.CUSTOMERNUM);
                pay.setP4_orderAmount("100");
                pay.setP12_serverCallback(secondUnionPay);
                orderId = heliDto.getProductId() + "T" + time + environment;
                String product = "采美订单" + orderId;
                URLEncoder.encode(product,"UTF-8");
                pay.setP8_goodsName(product);
                String desc=heliDto.getProductId().toString() + "," + BizType.getNumByType(heliDto.getUserType());
                URLEncoder.encode(desc,"UTF-8");
                pay.setP15_desc(desc);
            }else if ("vip".equals(payFlag)) {
                pay.setP3_customerNumber(Constant.CUSTOMERNUM);
                VipRecordBo record = payOrderMapper.getVipPackageRecord(heliDto.getVipRecordId());
                Double price = payOrderMapper.getVipPackagePriceById(record.getPackageId());
                pay.setP4_orderAmount(price.toString());
                pay.setP12_serverCallback(vipUnionPay);
                orderId = record.getUserId() + "R" + heliDto.getVipRecordId() + "V" + record.getPackageId() + "T" + time + environment;
                String product = "采美订单" + orderId;
                URLEncoder.encode(product,"UTF-8");
                pay.setP8_goodsName(product);
                String attach = record.getPackageId() + "," + record.getUserId() + "," + heliDto.getVipRecordId() + "," + BizType.getNumByType(heliDto.getUserType());
                URLEncoder.encode(attach,"UTF-8");
                pay.setP15_desc(attach);
            }else if("coupon".equals(payFlag)){
                pay.setP3_customerNumber(Constant.CUSTOMERNUM);
                Double couponPrice = payOrderMapper.getCouponPrice(heliDto.getCouponRecordId());
                pay.setP4_orderAmount(couponPrice.toString());
                pay.setP12_serverCallback(couponUnionPay);
                orderId = heliDto.getUserId() + "C" + heliDto.getCouponId() + "R" + heliDto.getCouponRecordId() + "T" + time + environment;
                String product = "采美订单" + orderId;
                URLEncoder.encode(product,"UTF-8");
                pay.setP8_goodsName(product);
                String attach = heliDto.getCouponId() + "," + heliDto.getUserId() + "," + heliDto.getCouponRecordId() + "," + BizType.getNumByType(heliDto.getPayType())+","+heliDto.getSource();
                URLEncoder.encode(attach,"UTF-8");
                pay.setP15_desc(attach);
            } else if ("authVip".equals(payFlag)) {
                pay.setP3_customerNumber(Constant.CUSTOMERNUM);
                VipRecordBo record = payOrderMapper.getAuthVipRecord(heliDto.getVipRecordId());
                Double price = payOrderMapper.getAuthVipPackagePrice(record.getPackageId());
                pay.setP4_orderAmount(price.toString());
                pay.setP12_serverCallback(authVipUnionPay);
                orderId = record.getUserId() + "A" + heliDto.getVipRecordId() + "V" + record.getPackageId() + "T" + time + environment;
                String product = "采美订单" + orderId;
                URLEncoder.encode(product,"UTF-8");
                pay.setP8_goodsName(product);
                String attach = record.getPackageId() + "," + record.getUserId() + "," + heliDto.getVipRecordId() + "," + BizType.getNumByType(heliDto.getUserType());
                URLEncoder.encode(attach,"UTF-8");
                pay.setP15_desc(attach);
            } else if ("order".equals(payFlag)) {
                OrderVo order = orderCommonMapper.getOrderByShopOrderId(heliDto.getShopOrderId());
                orderId = order.getOrderNo() + "T" + time + environment;
                pay.setP4_orderAmount(heliDto.getPayAmount());
                pay.setP12_serverCallback(unionPay);
                String product = "采美订单" + order.getOrderNo();
                URLEncoder.encode(product, "UTF-8");
                pay.setP8_goodsName(product);
                String attach = order.getOrderId() + "," + BizType.getNumByType(heliDto.getUserType()) + "," + heliDto.getShopOrderId();
                URLEncoder.encode(attach, "UTF-8");
                pay.setP15_desc(attach);
            }
            pay.setP1_bizType("OnlinePay");
            pay.setP2_orderId(orderId);
            pay.setP5_bankId(heliDto.getBankCode());
            pay.setP6_business(heliDto.getUserType());
            pay.setP7_timestamp(new SimpleDateFormat("yyyyMMddHHmmss").format(new Date()));
            pay.setP9_period("1");
            pay.setP10_periodUnit("hour");
            pay.setP11_callbackUrl(heliDto.getReturnUrl());
            pay.setP13_orderIp(headers.getFirst("X-CLIENT-IP"));
            pay.setP14_onlineCardType("DEBIT");
        } catch (UnsupportedEncodingException e) {
            log.error("网银支付设值错误=================>"+e);
        }
        return pay;
    }
}
